<?php
	class AnimationGroup {
		public $bitmaps;
		public $slices;
		public $anims;
		
		static public function evaluate($e) {
			if (!strlen($e = trim($e))) return false;
			eval("\$v = {$e};");
			return $v;
		}
		
		static $ganim_cached = array();
		
		static function get($file) {
			$ga = &$ganim_cached[$file];
			if (!isset($ga)) $ga = new static($file);
			return $ga;
		}
		
		function __construct($file) {
			$xml = simplexml_load_file($file);

			// Images
			foreach ($xml->xpath('image') as $e) {
				$this->bitmaps[(string)$e['id']] = Bitmap::fromFile((string)$e['file']);
			}

			// Slices
			foreach ($xml->xpath('cut') as $e) {
				$x  = static::evaluate($e['x']);
				$y  = static::evaluate($e['y']);
				$w  = static::evaluate($e['width']);
				$h  = static::evaluate($e['height']);
				$cx = static::evaluate($e['centerx']);
				$cy = static::evaluate($e['centery']);
				$img = $this->slices[(string)$e['id']] = $this->bitmaps[(string)$e['imageid']]->slice($x, $y, $w, $h);
				$img->cx = $cx;
				$img->cy = $cy;
				//$this->bitmaps[(string)$e['id']] = Bitmap::fromFile((string)$e['file']);
			}

			// Animation
			foreach ($xml->xpath('animation') as $xml_anim) {
				$anim = &$this->anims[(string)$xml_anim['id']];
				$time_mult = isset($xml_anim['timemultiplier']) ? static::evaluate($xml_anim['timemultiplier']) : 1;
				$ctime = 0;
				foreach ($xml_anim->xpath('frame') as $xml_frame) {
					$frame = &$anim['frames'][$ctime];
					foreach ($xml_frame->xpath('put') as $xml_put) {
						$frame['puts'][] = array(
							$this->slices[(string)$xml_put['cutid']],
							static::evaluate($xml_put['x']),
							static::evaluate($xml_put['y'])
						);
					}
					$frame['duration'] = static::evaluate($xml_frame['time']) * $time_mult;
					$ctime += $frame['duration'];
				}
				$anim['duration'] = $ctime;
			}
		}
		
		public function locateFrame($anim, $frame) {
			$a = &$this->anims[$anim];
			if (!isset($a)) return array();
			$frame %= $a['duration'];
			
			$f = &$a['frames'][0]['puts'];
			foreach (array_keys($a['frames']) as $t) {
				if ($t >= $frame) break;
				$f = &$a['frames'][$t]['puts'];
			}
			return $f;
		}
		
		function blit($screen, $anim, $frame, $x = 0, $y = 0) {
			foreach (static::locateFrame($anim, $frame) as $k => $put) {
				list($bmp, $_x, $_y) = $put;
				$bmp->blit($screen, $x + $_x, $y + $_y, 1, 0, 1);
			}
		}
	}

	class Sprite {
		public $ganim;
		public $anim, $frame;
		public $x, $y;
		public $segment;
		public $scenario;
		
		public function __construct(AnimationGroup $ganim, $x = 0, $y = 0, $scenario = null) {
			$this->ganim = $ganim;
			$this->anim = 'stop270';
			$this->frame = 0;
			$this->x = $x;
			$this->y = $y;
			$this->scenario = $scenario;
		}
	}
?>