<?php
	error_reporting(E_ALL | E_STRICT);
	require_once(dirname(__FILE__) . '/animation.php');
	require_once(dirname(__FILE__) . '/geometry.php');
	
	$screen = Screen::init(640, 480);
	$screen2 = new Bitmap($screen->w, $screen->h);

	/*
	$shader = new Shader("
		uniform sampler2D image;

		void main() {
			gl_FragColor = texture2D(image, vec2(gl_TexCoord[0]));
			//gl_FragColor = gl_FragColor * vec4(1, 0.8, 0.8, 1) + vec4(0.1, 0, 0, 0);
			gl_FragColor.a *= 1.0 - length(gl_TexCoord[0].xy - vec2(0.5, 0.5)) * 2;
		}
	");
	*/

	class Character extends Sprite {
		public $move_type = 'walk';
		public $last_dx = 0, $last_dy = 1;
		
		public function move(Point $d) {
			static $dir = array(
				'135', '090', '045',
				'180', '   ', '000',
				'225', '270', '315',
			);
			
			$offset = (Math::sign($d->y) + 1) * 3 + (Math::sign($d->x) + 1);
			$last_offset = (Math::sign($this->last_dy) + 1) * 3 + (Math::sign($this->last_dx) + 1);
			
			//echo $dir[$offset] . "\n";
			
			if ($dir[$offset] == '   ') {
				$this->anim = "stop{$dir[$last_offset]}";
			} else {
				$this->anim = "{$this->move_type}{$dir[$offset]}";
				$this->last_dx = $d->x;
				$this->last_dy = $d->y;
			}

			$this->frame += 1;

			$r = Rectangle::create($this->x - 16, $this->y - 8, 32, 16);

			$d = $this->scenario->gc->moveCollision($r, $d);
			
			$this->x += $d->x;
			$this->y += $d->y;
		}
	}

	class Scenario {
		public $gc;
		public $segments = array();
		public $charas = array();
		public $camera;
		public $following;
		
		public function __construct() {
			//$this->gc = new GeometryContainer();
			$this->gc = new GeometryContainerOptimized(new Point(32, 16));
			$this->gc->clear();
			$this->gc->add(Line::create(0, 0, 0, 480));
			$this->gc->add(Line::create(0, 0, 640, 100));
			$this->gc->add(Line::create(640, 0, 640, 480));
			$this->gc->add(Line::create(0, 480, 640, 480));
			
			$this->charas = array();
			$this->charas[] = $this->following = new Character(AnimationGroup::get('charas/reid.xml'), 320, 240, $this);
			$this->charas[] = new Character(AnimationGroup::get('charas/farah.xml'), 120, 120, $this);
			$this->camera = new Point(0, 0);
		}
		
		public function update() {
			$f = $this->following;
			$expected = new Point($f->x - 320, $f->y - 240);
			$this->camera = new Point(($expected->x + $this->camera->x) / 2, ($expected->y + $this->camera->y) / 2);
			Math::clamp(-100, 100, $this->camera->x);
			Math::clamp(-100, 100, $this->camera->y);
		}
		
		public function blit($screen) {
			//Sprite::blit($screen, $this->charas, 0, 0);
			
			$charas = $this->charas;
			
			usort($charas, function($a, $b) {
				return $a->y - $b->y;
			});
			
			$y = $this->camera->y;
			$x = $this->camera->x;
			
			foreach ($charas as $s) {
				$s->ganim->blit($screen, $s->anim, $s->frame, $s->x - $x, $s->y - $y);
			}
			
			foreach ($this->gc->lines as $s) {
				Draw::line($screen, $s->p1->x - $x, $s->p1->y - $y, $s->p2->x - $x, $s->p2->y - $y, 6, array(1, 1, 1, 0.5));
			}
		}
	}
	
	$scenario = new Scenario();

	$font = Font::fromName('Arial Unicode MS', 18);
	$font->style |= Font::STYLE_BOLD;
	
	$mpos_1 = null;
	$mpos_2 = null;
	
	while (!Keyboard::pressed(Keyboard::ESC)) {
		$dy = $dx = 0;
		if (Keyboard::down(Keyboard::LEFT )) $dx -= 3;
		if (Keyboard::down(Keyboard::RIGHT)) $dx += 3;
		if (Keyboard::down(Keyboard::UP   )) $dy -= 3;
		if (Keyboard::down(Keyboard::DOWN )) $dy += 3;
		
		$scenario->following->move(Point::create($dx, $dy));
		$scenario->update();
		
		list($mx, $my) = Mouse::position();

		$mpos_2 = new Point($mx + $scenario->camera->x, $my + $scenario->camera->y);
		
		if (Mouse::pressed(Mouse::LEFT)) {
			if ($mpos_1 !== null) {
				$scenario->gc->add(new Line($mpos_1, $mpos_2));
			}			
			$mpos_1 = $mpos_2;
			//printf("click");
		}
		if (Mouse::pressed(Mouse::RIGHT)) {
			$mpos_1 = null;
		}
		
		if (Keyboard::pressed(Keyboard::SPACE)) {
			foreach ($scenario->charas as $chara) {
				if ($chara != $scenario->following) {
					$scenario->following = $chara;
					break;
				}
			}
		}
		
		$screen->clear(0, 0.4, 0, 1);

		$scenario->blit($screen);

		if ($mpos_1 !== null) {
			Draw::line(
				$screen,
				$mpos_1->x - $scenario->camera->x, $mpos_1->y - $scenario->camera->y,
				$mpos_2->x - $scenario->camera->x, $mpos_2->y - $scenario->camera->y,
				4, array(0, 0, 1, 0.4)
			);
		}
		
		$text = sprintf(
			"Position: %d, %d\nLines: %d\nChecked lines: %d\n",
			$scenario->following->x, $scenario->following->y,
			count($scenario->gc->lines),
			$scenario->gc->lastLinesInRect
		);
		$font->blit($screen, $text, 9, 9, array(0, 0, 0, 0.5));
		$font->blit($screen, $text, 8, 8, array(1, 1, 1, 1));

		Screen::frame();
	}
?>